unit w_HashedStringListTest;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ExtCtrls,
  System.Generics.Collections;

type
  Tf_HashedStringListTest = class(TForm)
    m_Output: TMemo;
    p_Bottom: TPanel;
    b_Execute: TButton;
    b_Exit: TButton;
    procedure b_ExecuteClick(Sender: TObject);
    procedure b_ExitClick(Sender: TObject);
  private
    procedure DoTiming(sl: TStringList); overload;
    procedure DoTiming(sl: TDictionary < string, Integer > ); overload;
  public
  end;

var
  f_HashedStringListTest: Tf_HashedStringListTest;

implementation

{$R *.dfm}

uses
  System.IniFiles,
  System.Diagnostics;

procedure Tf_HashedStringListTest.DoTiming(sl: TDictionary < string, Integer > );
const
  CYCLES = 10000;
var
  k: Integer;
  i: Integer;
  j: Integer;
  sw: TStopwatch;
  s: string;
  v: Integer;
begin
  sw := TStopwatch.StartNew;
  for k := 1 to CYCLES do begin
    for i := Ord('A') to Ord('Z') do begin
      for j := Ord('A') to Ord('Z') do begin
        s := chr(i) + chr(j);
        if not sl.TryGetValue(s, v) then
          sl.Add(s, i * 100 + j);
      end;
    end;
  end;
  sw.Stop;
  m_Output.Lines.Add(sl.Count.ToString + ': Add: ' + sw.Elapsed.ToString);

  sw.Reset;
  sw.Start;
  for k := 1 to CYCLES do begin
    for i := Ord('A') to Ord('Z') do begin
      for j := Ord('A') to Ord('Z') do begin
        s := chr(i) + chr(j);
        sl.Items[s];
      end;
    end;
  end;
  m_Output.Lines.Add(sl.Count.ToString + ': IndexOf: ' + sw.Elapsed.ToString);
//  for i := 0 to sl.Count - 2 do
//    if sl [i] > sl[i + 1] then
//      m_Output.Lines.Add(Format('Error: %s > %s', [sl[i], sl[i + 1]]));
end;

procedure Tf_HashedStringListTest.b_ExecuteClick(Sender: TObject);
var
  sl: TStringList;
  dict: TDictionary<string, Integer>;
begin
  sl := TStringList.Create;
  try
    DoTiming(sl);
  finally
    FreeAndNil(sl);
  end;

  sl := THashedStringList.Create;
  try
    DoTiming(sl);
  finally
    FreeAndNil(sl);
  end;

  dict := TDictionary < string, Integer > .Create;
  try
    DoTiming(dict);
  finally
    FreeAndNil(dict);
  end;
end;

procedure Tf_HashedStringListTest.b_ExitClick(Sender: TObject);
begin
  Close;
end;

procedure Tf_HashedStringListTest.DoTiming(sl: TStringList);
const
  CYCLES = 10000;
var
  k: Integer;
  i: Integer;
  j: Integer;
  sw: TStopwatch;
  s: string;
  Idx: Integer;
begin
  sl.Sorted := True;
  sl.CaseSensitive := True;
  sl.Duplicates := dupError;
  sw := TStopwatch.StartNew;
  sl.BeginUpdate;
  for k := 1 to CYCLES do begin
    for i := Ord('A') to Ord('Z') do begin
      for j := Ord('A') to Ord('Z') do begin
        s := chr(i) + chr(j);
        if not sl.Find(s, Idx) then
          sl.AddObject(s, Pointer(i * 100 + j));
      end;
    end;
  end;
  sl.EndUpdate;
  sw.Stop;
  m_Output.Lines.Add(sl.Count.ToString + ': Add: ' + sw.Elapsed.ToString);

  sw.Reset;
  sw.Start;
  for k := 1 to CYCLES do begin
    for i := Ord('A') to Ord('Z') do begin
      for j := Ord('A') to Ord('Z') do begin
        s := chr(i) + chr(j);
        sl.IndexOf(s);
      end;
    end;
  end;
  m_Output.Lines.Add(sl.Count.ToString + ': IndexOf: ' + sw.Elapsed.ToString);
  for i := 0 to sl.Count - 2 do
    if sl[i] > sl[i + 1] then
      m_Output.Lines.Add(Format('Error: %s > %s', [sl[i], sl[i + 1]]));
end;

end.

